"use strict";

var WavEncoder = require("wav-encoder");
var EncoderUtils = require("./utils/EncoderUtils");

var encoders = {};

/**
 * @param {string}    type
 * @return {function}
 */
function get(type) {
  return encoders[type] || null;
}

/**
 * @param {string}   type
 * @param {function} fn
 */
function set(type, fn) {
  /* istanbul ignore else */
  if (typeof fn === "function") {
    encoders[type] = fn;
  }
}

/**
 * @param {AudioData} audioData
 * @param {object}    opts
 * @return {Promise<ArrayBuffer>}
 */
function encode(audioData) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var type = opts.type || "wav";
  var encodeFn = encoders[type];

  if (typeof encodeFn !== "function") {
    return Promise.reject(new TypeError("Encoder does not support the audio format: " + type));
  }

  return EncoderUtils.encode(encodeFn, audioData, opts);
}

set("wav", WavEncoder.encode);

module.exports = { get: get, set: set, encode: encode };